# -*- coding: utf-8 -*-
# ###
# Copyright (c) 2015, Rice University
# This software is subject to the provisions of the GNU Affero General
# Public License version 3 (AGPLv3).
# See LICENCE.txt for details.
# ###
import os
import io
import logging
import socket
import unittest
from tempfile import mkstemp

from pyramid import testing


class UtilsTests(unittest.TestCase):

    def test_local_settings(self):
        prefix = 'xyz'
        settings = {
            '{}.foo'.format(prefix): 'bar',
            '{}.bar'.format(prefix): 'foo',
            'oof': 'arb',
            }
        expected = {
            'foo': 'bar',
            'bar': 'foo',
            }
        # Test the utility...
        from .utils import local_settings
        self.assertEqual(expected, local_settings(settings, prefix))


CONFIG = """\
version: 1

formatters:
  generic:
    format    : '%(levelname)-5.5s;.;%(message)s'
  contextualized:
    format    : '%(levelname)-5.5s;.;%(hostname)s;.;%(message)s'

filters:
  context:
    ()        : pyramid_sawing.filters.ContextFilter

handlers:
  logio:
    class     : logging.StreamHandler
    level     : INFO
    formatter : generic
    stream    : 'ext://pyramid_sawing.tests.logio'
  logio_with_context:
    class     : logging.StreamHandler
    level     : INFO
    formatter : contextualized
    filters   : [context]
    stream    : 'ext://pyramid_sawing.tests.logio'

loggers:
  pyramid_sawing.tests:
    level     : INFO
    handlers  : [logio_with_context]
    propagate : 0

root:
  level       : NOTSET
  handlers    : [logio]
"""
logio = None


class IncludePluginTestCase(unittest.TestCase):

    def setUp(self):
        global logio
        logio = io.BytesIO()

        # Put the config somewhere.
        self.logging_config_filepath = mkstemp()[1]
        with open(self.logging_config_filepath, 'w') as f:
            f.write(CONFIG)
        # Settings...
        self.settings = {
            'pyramid_sawing.file': self.logging_config_filepath,
            }
        self.addCleanup(os.remove, self.logging_config_filepath)

    def tearDown(self):
        global logio
        logio = None

    def test_logging(self):
        """Test that the includeme function loads the logging config."""
        from .main import includeme
        with testing.testConfig(settings=self.settings) as config:
            includeme(config)

        # Set up the loggers
        root = logging.getLogger()
        local = logging.getLogger('pyramid_sawing.tests')

        # Try logging...
        root_log_msg = 'O.o'
        root.error(root_log_msg)
        local_log_msg = '>,<'
        local.info(local_log_msg)

        global logio
        logio.seek(0)
        log_lines = logio.read().split('\n')
        self.assertEqual(len(log_lines), 3)

        # Check the root message...
        parsed_root_msg = log_lines[0].split(';.;')
        self.assertEqual(parsed_root_msg, ['ERROR', root_log_msg])
        parsed_local_msg = log_lines[1].split(';.;')
        self.assertEqual(parsed_local_msg,
                         ['INFO ', socket.gethostname(), local_log_msg])



# -*- coding: utf-8 -*-
# ###
# Copyright (c) 2015, Rice University
# This software is subject to the provisions of the GNU Affero General
# Public License version 3 (AGPLv3).
# See LICENCE.txt for details.
# ###
from logging.config import dictConfig

import yaml

from .utils import local_settings

__all__ = (
    'includeme',
    )

PROJECT = 'pyramid_sawing'
PREFIX = PROJECT


def includeme(config):
    """Pyramid pluggable and discoverable function."""
    global_settings = config.registry.settings
    settings = local_settings(global_settings, PREFIX)

    try:
        file = settings['file']
    except KeyError:
        raise KeyError("Must supply '{}.file' configuration value "
                       "in order to configure logging via '{}'."
                       .format(PREFIX, PROJECT))

    with open(file, 'r') as f:
        logging_config = yaml.load(f)

    dictConfig(logging_config)



# -*- coding: utf-8 -*-
import os
import sys
from setuptools import setup, find_packages


IS_PY3 = sys.version_info > (3,)

install_requires = (
    'pyramid',
    'PyYAML',
    )
tests_require = [
    ]
extras_require = {
    'test': tests_require,
    }
description = "Pyramid plugin for YAML logging configuration."
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as readme:
    README = readme.read()
with open(os.path.join(here, 'CHANGES.rst')) as changes:
    CHANGELOG = changes.read()
changelog_header = """\
Change Log
==========
"""
long_description = '\n\n'.join([README, changelog_header, CHANGELOG])

setup(
    name='pyramid_sawing',
    version='1.0.0',
    author='Connexions team',
    author_email='info@cnx.org',
    url="https://github.com/connexions/pyramid_sawing",
    license='AGPL, See also LICENSE.txt',
    description=description,
    long_description=long_description,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=extras_require,
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'pyramid_sawing': ['*.yaml'],
        'pyramid_sawing.tests': ['*.yaml'],
        },
    test_suite='pyramid_sawing.tests',
    entry_points={},
    )



# -*- coding: utf-8 -*-
# ###
# Copyright (c) 2015, Rice University
# This software is subject to the provisions of the GNU Affero General
# Public License version 3 (AGPLv3).
# See LICENCE.txt for details.
# ###
import logging
import socket


__all__ = ('ContextFilter')


class ContextFilter(logging.Filter):
    """Provides context specific filter values.
    See also https://docs.python.org/3/library/logging.html#filter-objects
    """
    hostname = socket.gethostname()

    def filter(self, record):
        record.hostname = ContextFilter.hostname
        return True
